/**
 * Copyright  World Wide Web Consortium, (Massachusetts Institute of
 * Technology, Institut National de Recherche en Informatique et en
 * Automatique, Keio University).
 *
 * All Rights Reserved.
 *
 * @author	Sergey Melnik <melnik@db.stanford.edu>
 */

package org.w3c.rdf.model.impl;

import org.w3c.rdf.model.*;
import org.w3c.rdf.util.RDFDigestUtil;
import org.w3c.tools.crypt.*;
import java.io.*;

public final class TripleImpl implements Triple, Digestable {

  Resource	pred;
  Resource	subj;
  RDFNode	obj;
  Digest        digest;

  /**
   * The parameters to constructor are instances of classes
   * and not just strings
   */
  public TripleImpl (Resource pred, Resource subj, RDFNode obj) {
    this.pred = pred;
    this.subj = subj;
    this.obj = obj;
  }

  public Resource predicate () {
    return pred;
  }

  public Resource subject () {
    return subj;
  }

  public RDFNode object () {
    return obj;
  }

  /**
   * @return four most significant bytes of the digest
   */
  public int hashCode() {

    return DigestUtil.getHashCode(getDigest());
      
    /*
    if(hashCode == 0)
      hashCode = subj.hashCode() ^ pred.hashCode() ^ obj.hashCode();
    return hashCode;
    */
  }

  /**
   * Properties: digest can be constructed given the digests of the pred, subj, obj
   * Permutations of the s, p, o return different digests
   */

  public Digest getDigest() {

    if(digest == null) {
      digest = RDFDigestUtil.getTripleDigest(this);
    }
    return digest;
  }

  public String getURI() {

    return "urn:rdf:statement:" + DigestUtil.toHexString(getDigest());
  }

  public String getLabel() {
    return getURI();
  }

  public String toString() {

    String objStr = obj instanceof Literal ? "literal(\"" + obj + "\")" : "\"" + obj + "\"";
    return "triple(\"" + subj + "\", \"" + pred + "\", " + objStr + ")";
  }

  public boolean equals (Object that) {
    
    if (this == that) {
      return true;
    }
    if (that == null || !(that instanceof Triple)) {
      return false;
    }

    if(that instanceof Digestable) {
      return DigestUtil.equal(getDigest(), ((Digestable)that).getDigest());
    }

    Triple t = (Triple)that;
    
    return
      subj.equals(t.subject()) &&
      pred.equals(t.predicate()) &&
      obj.equals(t.object());
  }

  public static void main(String[] args) {

    if(args.length == 0) {
      System.err.println("TripleImpl <subject> <predicate> <object> [literal]");
      System.exit(1);
    }

    RDFNode object;
    if(args.length > 3)
      object =  new LiteralImpl(args[2]);
    else
      object = new ResourceImpl(args[2]);

    Triple t = new TripleImpl(new ResourceImpl(args[0]),
			      new ResourceImpl(args[1]),
			      object);

    System.err.println("Triple URI is: " + t.getURI() + ", hashCode: " + t.hashCode());
  }
}
